"""Module that defines fitting functions."""

from typing import List
import numpy as np

def linear(vectors: List[np.ndarray], target: np.ndarray):
    """Simple least square minimization fitting."""
    matrix = np.vstack(vectors).T
    coefficients, _, _, _ = np.linalg.lstsq(matrix, target, rcond=None)
    return np.array(coefficients, dtype=np.float64)

def quasi_time_reversible():
    """Time reversible least square minimization fitting."""

def linear_combination(vectors: List[np.ndarray], coefficients: np.ndarray) -> np.ndarray:
    """Given a set of vectors (or matrices) and the corresponding
    coefficients, build their linear combination."""
    result = np.zeros(vectors[0].shape, dtype=np.float64)
    for coeff, vector in zip(coefficients, vectors):
        result += vector*coeff
    return result
