import pytest
import os
import sys
import numpy as np

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from gext.buffer import CircularBuffer

def test_buffer():

    shape = (5, 5)
    buffer_size = 10
    nframes = 20

    buffer = CircularBuffer(buffer_size, shape)

    # partial load
    for i in range(buffer_size // 2):
        matrix = np.full(shape, i)
        buffer.push(matrix)

    # do some tests on a partially filled buffer
    with pytest.raises(ValueError):
        buffer.get(buffer_size+1)

    with pytest.raises(ValueError):
        buffer.get(buffer_size//2 + 1)

    with pytest.raises(ValueError):
        buffer.get(-1)

    assert len(buffer.get(0)) == 0

    for m in [buffer_size//2, buffer_size//2-1, buffer_size//2-2]:
        matrices = buffer.get(m)
        for value, matrix in zip(list(range(buffer_size//2 - m, buffer_size//2)), matrices):
            assert matrix[0,0] == value

    # finish the loading
    for i in range(buffer_size // 2, nframes):
        matrix = np.full(shape, i)
        buffer.push(matrix)

    # do some tests on a fully filled buffer
    with pytest.raises(ValueError):
        buffer.get(buffer_size+1)

    buffer.get(buffer_size//2 + 1)

    with pytest.raises(ValueError):
        buffer.get(-1)

    assert len(buffer.get(0)) == 0

    for m in [buffer_size, buffer_size-1, buffer_size-2]:
        matrices = buffer.get(m)
        for value, matrix in zip(list(range(nframes - m, nframes)), matrices):
            assert matrix[0,0] == value

def test_buffer_manual():
    shape = (5, 5)
    buffer = CircularBuffer(6, shape)

    for i in range(6):
        matrix = np.full(shape, i)
        buffer.push(matrix)

    matrices = buffer.get(6)
    for matrix, value in zip(matrices, range(6)):
        assert matrix[0, 0] == value
